@@ -1,16 +1,15 @@
 ; -*- Lisp -*-
 (in-package :maxima)
 
-(defparameter *autoconf-variables-set* "@variables_set@")
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.42.2")
+(defparameter *autoconf-version* "5.43.0")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
-(defparameter *autoconf-host* "powerpc-unknown-openbsd6.5")
+(defparameter *autoconf-host* "x86_64-unknown-openbsd6.6")
 ;; This variable is kept for backwards compatibiliy reasons:
 ;; We seem to be in the fortunate position that we sometimes need to check for windows.
 ;; But at least until dec 2015 we didn't need to check for a specific windows flavour.
@@ -19,7 +18,7 @@
 (defparameter *autoconf-ld-flags* "")
 
 ;; This will be T if this was a lisp-only build
-(defparameter *autoconf-lisp-only-build* (eq t '@lisp_only_build@))
+(defparameter *autoconf-lisp-only-build* (eq t 'nil))
  
-(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.42.2/maxima-5.42.2")
+(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.43.0/maxima-5.43.0")
 (defparameter *maxima-default-layout-autotools* "true")
