@@ -56,27 +56,27 @@
 ;; in case a file-name is supplied, the output will be sent
 ;; (perhaps appended) to that file.
 
-(declare-top (special lop rop ccol $gcprint texport $labels $inchar vaxima-main-dir))
+(declare-top (special lop rop $labels $inchar))
 
 (defvar *tex-environment-default* '("$$" . "$$"))
 
-(defun $set_tex_environment_default (env-open env-close)
+(defmfun $set_tex_environment_default (env-open env-close)
   (setq env-open ($sconcat env-open))
   (setq env-close ($sconcat env-close))
   (setq *tex-environment-default* `(,env-open . ,env-close))
   ($get_tex_environment_default))
 
-(defun $get_tex_environment_default ()
+(defmfun $get_tex_environment_default ()
   `((mlist) ,(car *tex-environment-default*) ,(cdr *tex-environment-default*)))
 
-(defun $set_tex_environment (x env-open env-close)
+(defmfun $set_tex_environment (x env-open env-close)
   (setq env-open ($sconcat env-open))
   (setq env-close ($sconcat env-close))
   (if (getopr x) (setq x (getopr x)))
   (setf (get x 'tex-environment) `(,env-open . ,env-close))
   ($get_tex_environment x))
 
-(defun $get_tex_environment (x)
+(defmfun $get_tex_environment (x)
   (if (getopr x) (setq x (getopr x)))
   (let ((e (get-tex-environment x)))
     `((mlist) ,(car e) ,(cdr e))))
@@ -106,6 +106,8 @@
 (defmspec $tex(l) ;; mexplabel, and optional filename or stream
   ;;if filename or stream supplied but 'nil' then return a string
   (let ((args (cdr l)))
+    (unless (member (length args) '(1 2))
+      (wna-err '$tex))
     (cond ((and (cdr args) (null (cadr args)))
 	   (let ((*standard-output* (make-string-output-stream)))
 	     (apply 'tex1  args)
@@ -114,21 +116,20 @@
 	   )
 	  (t (apply 'tex1  args)))))
 
-(defun quote-% (sym)
+(defun quote-chars (sym ch-str)
   (let* ((strsym (string sym))
-         (pos (position-if #'(lambda (c) (find c "$%&_")) strsym)))
+         (pos (position-if #'(lambda (c) (find c ch-str)) strsym)))
     (if pos
       (concatenate 'string (subseq strsym 0 pos) "\\" (subseq strsym pos (1+ pos))
-                           (quote-% (subseq strsym (1+ pos))))
+                           (quote-chars (subseq strsym (1+ pos)) ch-str))
       strsym)))
 
+(defun quote-% (sym)
+  (quote-chars sym "$%&_"))
+
 (defun tex1 (mexplabel &optional filename-or-stream) ;; mexplabel, and optional filename or stream
-  (prog (mexp  texport $gcprint ccol x y itsalabel need-to-close-texport)
-     ;; $gcprint = nil turns gc messages off
-     (setq ccol 1)
-     (cond ((null mexplabel)
-	    (displa " No eqn given to TeX")
-	    (return nil)))
+  (prog (mexp  texport x y itsalabel need-to-close-texport)
+     (reset-ccol)
      ;; collect the file-name, if any, and open a port if needed
      (setq filename-or-stream (meval filename-or-stream))
      (setq texport
@@ -188,8 +189,8 @@
 	   (t 
 	    (if mexplabel (setq mexplabel (quote-% mexplabel)))
 					; display the expression for TeX now:
-        (myprinc (car (get-tex-environment mexp)))
-	    (mapc #'myprinc
+        (myprinc (car (get-tex-environment mexp)) texport)
+	    (mapc #'(lambda (x) (myprinc x texport))
 		  ;;initially the left and right contexts are
 		  ;; empty lists, and there are implicit parens
 		  ;; around the whole expression
@@ -214,22 +215,20 @@
 ;;-              that a value is all printed on one line (and not divided
 ;;-              by the crazy top level os routines)
 
-(defun myprinc (chstr)
-  (prog (chlst)
-     (cond ((and (> (+ (length (setq chlst (exploden chstr))) ccol) 70.)
-                 (or (stringp chstr) (equal chstr '| |)))
-	    (terpri texport)      ;would have exceeded the line length
-	    (setq ccol 1.)
-	    (myprinc " "))) ; lead off with a space for safetyso we split it up.			
-     (do ((ch chlst (cdr ch))
-	  (colc ccol (1+ colc)))
-	 ((null ch) (setq ccol colc))
-       (write-char (car ch) texport))))
+(let ((ccol 1))
+  (defun reset-ccol () (setq ccol 1))
 
-(defun myterpri nil
-  (cond (texport (terpri texport))
-	(t (mterpri)))
-  (setq ccol 1))
+  (defun myprinc (chstr &optional (texport nil))
+    (prog (chlst)
+       (cond ((and (> (+ (length (setq chlst (exploden chstr))) ccol) 70.)
+                   (or (stringp chstr) (equal chstr '| |)))
+	      (terpri texport)      ;would have exceeded the line length
+	      (setq ccol 1.)
+	      (myprinc " " texport))) ; lead off with a space for safetyso we split it up.			
+       (do ((ch chlst (cdr ch))
+	    (colc ccol (1+ colc)))
+	   ((null ch) (setq ccol colc))
+         (write-char (car ch) texport)))))
 
 (defun tex (x l r lop rop)
   ;; x is the expression of interest; l is the list of strings to its
@@ -253,61 +252,70 @@
                       ((stringp x)
                        (tex-string (quote-% (if $stringdisp (concatenate 'string "``" x "''") x))))
                       ((characterp x) (tex-char x))
+		      ((not ($mapatom x))
+		       (let ((x (if (member (marray-type x) '(array hash-table $functional))
+				    ($sconcat x)
+				  (format nil "~A" x))))
+			 (tex-string (quote-chars (if $stringdisp (concatenate 'string "``" x "''") x)
+						  "#$%&_"))))
+			 
 		      (t (tex-stripdollar (or (get x 'reversealias) x)))))
 	  r))
 
 (defun tex-string (x)
   (cond ((equal x "") "")
 	((eql (elt x 0) #\\) x)
-	(t (concatenate 'string "\\mbox{{}" x "{}}"))))
+	(t (concatenate 'string "\\mbox{ " x " }"))))
 
 (defun tex-char (x)
   (if (eql x #\|) "\\mbox{\\verb/|/}"
       (concatenate 'string "\\mbox{\\verb|" (string x) "|}")))
 
-(defvar *tex-translations* nil)
-;; '(("ab" . "a")("x" . "x")) would cause  AB12 and X3 C4 to print a_{12} and x_3 C_4
-
 ;; Read forms from file F1 and output them to F2
 (defun tex-forms (f1 f2 &aux tem (eof *mread-eof-obj*))
   (with-open-file (st f1)
     (loop while (not (eq (setq tem (mread-raw st eof)) eof))
 	   do (tex1 (third tem) f2))))
 
-(defun tex-stripdollar (x)
-  (let ((s (maybe-invert-string-case (symbol-name (tex-stripdollar0 x)))))
-    (if (> (length s) 1)
-      (concatenate 'string "{\\it " s "}")
-      s)))
+;; Detect and extract groups of trailing digits, e.g. foo_mm_nn.
+;; and then punt foo[mm, nn] to TEX-ARRAY.
+;; Otherwise, treat SYM as a simple symbol.
 
-(defun tex-stripdollar0 (sym &aux )
-  (or (symbolp sym) (return-from tex-stripdollar0  sym))
-  (let* ((pname (quote-% (stripdollar sym)))
-	 (l (length pname))
-	 (begin-sub
-	  (loop for i downfrom (1- l)
-		 when (not (digit-char-p (aref pname i)))
-		 do (return (1+ i))))
-	 (tem  (make-array (+ l 4) :element-type ' #.(array-element-type "abc") :fill-pointer 0)))
-    (loop for i below l
-	   do
-	   (cond ((eql i begin-sub)
-		  (let ((a (assoc tem  *tex-translations* :test 'equal)))
-		    (cond (a
-			   (setq a (cdr a))
-			   (setf (fill-pointer tem) 0)
-			   (loop for i below (length a)
-				  do
-				  (vector-push (aref a i) tem)))))
-		  (vector-push #\_ tem)
-		  (unless (eql i (- l 1))
-		    (vector-push #\{ tem)
-		    (setq begin-sub t))))
-		  (vector-push (aref pname i) tem)
-	   finally
-	   (cond ((eql begin-sub t)
-		  (vector-push #\} tem))))
-    (intern tem)))
+(defun tex-stripdollar (sym)
+  (let
+    ((nn-list (extract-trailing-digits (symbol-name sym))))
+    (if nn-list
+      ;; SYM matches foo_mm_nn.
+      (apply #'concatenate 'string (tex-array `((,(intern (first nn-list)) 'array) ,@(rest nn-list)) nil nil))
+      ;; SYM is a simple symbol.
+      (let ((s (maybe-invert-string-case (quote-% (stripdollar sym)))))
+        (if (> (length s) 1)
+          (concatenate 'string "{\\it " s "}")
+          s)))))
+
+;; Given a string foo_mm_nn, return foo, mm, and nn,
+;; where mm and nn are integers (not strings of digits).
+;; Return NIL if argument doesn't have trailing digits.
+
+(defun extract-trailing-digits (s)
+  (let (nn-list)
+    ;; OK (loop while (funcall #.(maxima-nregex::regex-compile "[^_](__*)([0-9][0-9]*)$") s)
+    ;; NOPE (loop while (funcall #.(maxima-nregex::regex-compile "[^0-9_](_*)([0-9][0-9]*)$") s)
+    (loop with nn-string while
+          (or (and
+                (funcall #.(maxima-nregex::regex-compile "[^_](__*)([0-9][0-9]*)$") s)
+                (let*
+                  ((group-_ (aref maxima-nregex::*regex-groups* 1))
+                   (group-nn (aref maxima-nregex::*regex-groups* 2)))
+                  (setq nn-string (subseq s (first group-nn) (second group-nn)))
+                  (setq s (subseq s 0 (first group-_)))))
+              (and
+                (funcall #.(maxima-nregex::regex-compile "[^_]([0-9][0-9]*)$") s)
+                (let* ((group-nn (aref maxima-nregex::*regex-groups* 1)))
+                  (setq nn-string (subseq s (first group-nn) (second group-nn)))
+                  (setq s (subseq s 0 (first group-nn))))))
+          do (push (parse-integer nn-string) nn-list))
+    (and nn-list (cons s nn-list))))
 
 (defun strcat (&rest args)
   (apply #'concatenate 'string (mapcar #'string args)))
@@ -372,8 +380,7 @@
   (tex (cadr x) (append l (texsym (caar x))) r (caar x) rop))
 
 (defun tex-infix (x l r)
-  ;; check for 2 args
-  (if (or (null (cddr x)) (cdddr x)) (wna-err (caar x)))
+  (twoargcheck x)
   (setq l (tex (cadr x) l nil lop (caar x)))
   (tex (caddr x) (append l (texsym (caar x))) r (caar x) rop))
 
@@ -445,6 +452,8 @@
 
 (defprop mlist tex-matchfix tex)
 (defprop mlist (("\\left[ ")" \\right] ") texsym)
+(setf (get '%mlist 'tex) (get 'mlist 'tex))
+(setf (get '%mlist 'texsym) (get 'mlist 'texsym))
 
 ;;absolute value
 (defprop mabs tex-matchfix tex)
@@ -473,6 +482,10 @@
  (tex (cadr x) (append l '("\\Gamma\\left(")) (append '("\\right)") r) 'mparen 'mparen))
 
 (defprop $%gamma "\\gamma" texword)
+(defprop %gamma_incomplete "\\Gamma" texword)
+(defprop %gamma_incomplete_regularized "Q" texword)
+(defprop %gamma_incomplete_generalized "\\Gamma" texword)
+(defprop $gamma_incomplete_lower "\\gamma" texword)
 (defprop $delta "\\delta" texword)
 (defprop $epsilon "\\varepsilon" texword)
 (defprop $zeta "\\zeta" texword)
@@ -555,61 +568,65 @@
     (let ((r (exploden number)))
       (member 'e r :test #'string-equal))))
 
+(defvar *tex-mexpt-trig-like-fns* '(%sin %cos %tan %sinh %cosh %tanh %asin %acos %atan %asinh %acosh %atanh))
+(defun tex-mexpt-trig-like-fn-p (f)
+  (member f *tex-mexpt-trig-like-fns*))
+(defun maybe-tex-mexpt-trig-like (x l r)
+  ;; here is where we have to check for f(x)^b to be displayed
+  ;; as f^b(x), as is the case for sin(x)^2 .
+  ;; which should be sin^2 x rather than (sin x)^2 or (sin(x))^2.
+  ;; yet we must not display (a+b)^2 as +^2(a,b)...
+  ;; or (sin(x))^(-1) as sin^(-1)x, which would be arcsine x
+  (let*
+      ((fx (cadr x)) ; this is f(x)
+       (f (and (not (atom fx)) (atom (caar fx)) (caar fx))) ; this is f [or nil]
+       (bascdr (and f (cdr fx))) ; this is (x) [maybe (x,y..), or nil]
+       (expon (caddr x)) ;; this is the exponent
+       (doit (and
+	      f ; there is such a function
+	      (tex-mexpt-trig-like-fn-p f) ; f is trig-like
+	      (member (get-first-char f) '(#\% #\$) :test #'char=) ;; insist it is a % or $ function
+	      (not (member 'array (cdar fx) :test #'eq)) ; fix for x[i]^2
+	      (or (and (atom expon) (not (numberp expon))) ; f(x)^y is ok
+		  (and (atom expon) (numberp expon) (> expon 0))))))
+                                        ; f(x)^3 is ok, but not f(x)^-1, which could
+                                        ; inverse of f, if written f^-1 x
+                                        ; what else? f(x)^(1/2) is sqrt(f(x)), ??
+    (cond (doit
+	   (setq l (tex `((mexpt) ,f ,expon) l nil 'mparen 'mparen))
+	   (if (and (null (cdr bascdr))
+		    (eq (get f 'tex) 'tex-prefix))
+	       (setq r (tex (car bascdr) nil r f 'mparen))
+	       (setq r (tex (cons '(mprogn) bascdr) nil r 'mparen 'mparen)))
+	   (append l r))
+	  (t nil))) ; won't doit. fall through
+  )
+
 ;; insert left-angle-brackets for mncexpt. a^<n> is how a^^n looks.
 (defun tex-mexpt (x l r)
-  (let((nc (eq (caar x) 'mncexpt)))	; true if a^^b rather than a^b
-    ;; here is where we have to check for f(x)^b to be displayed
-    ;; as f^b(x), as is the case for sin(x)^2 .
-    ;; which should be sin^2 x rather than (sin x)^2 or (sin(x))^2.
-    ;; yet we must not display (a+b)^2 as +^2(a,b)...
-    ;; or (sin(x))^(-1) as sin^(-1)x, which would be arcsine x
+  (let((nc (eq (caar x) 'mncexpt))) ; true if a^^b rather than a^b
     (cond ;; this whole clause
       ;; should be deleted if this hack is unwanted and/or the
       ;; time it takes is of concern.
       ;; it shouldn't be too expensive.
       ((and (eq (caar x) 'mexpt)      ; don't do this hack for mncexpt
-	    (let*
-		((fx (cadr x))		; this is f(x)
-		 (f (and (not (atom fx)) (atom (caar fx)) (caar fx))) ; this is f [or nil]
-		 (bascdr (and f (cdr fx))) ; this is (x) [maybe (x,y..), or nil]
-		 (expon (caddr x)) ;; this is the exponent
-		 (doit (and
-			f		; there is such a function
-			(member (getcharn f 1) '(#\% #\$)) ;; insist it is a % or $ function
-            (not (member 'array (cdar fx) :test #'eq))	; fix for x[i]^2
-					; Jesper Harder <harder@ifa.au.dk>
-			(not (member f '(%sum %product %derivative %integrate %at
-					      %lsum %limit $pderivop) :test #'eq)) ;; what else? what a hack...
-			(or (and (atom expon) (not (numberp expon))) ; f(x)^y is ok
-			    (and (atom expon) (numberp expon) (> expon 0))))))
-					; f(x)^3 is ok, but not f(x)^-1, which could
-					; inverse of f, if written f^-1 x
-					; what else? f(x)^(1/2) is sqrt(f(x)), ??
-	      (cond (doit
-		     (setq l (tex `((mexpt) ,f ,expon) l nil 'mparen 'mparen))
-		     (if (and (null (cdr bascdr))
-			      (eq (get f 'tex) 'tex-prefix))
-			 (setq r (tex (car bascdr) nil r f 'mparen))
-			 (setq r (tex (cons '(mprogn) bascdr) nil r 'mparen 'mparen))))
-		    (t nil)))))		; won't doit. fall through
-      (t (setq l (cond ((or ($bfloatp (cadr x)) (and (numberp (cadr x))
-			     (numneedsparen (cadr x))))
-            ; ACTUALLY THIS TREATMENT IS NEEDED WHENEVER (CAAR X) HAS GREATER BINDING POWER THAN MTIMES ...
-			(tex (cadr x) (append l '("\\left(")) '("\\right)") lop
-			     (caar x)))
-		       (t (tex (cadr x) l nil lop (caar x))))
-	       r (if (mmminusp (setq x (nformat (caddr x))))
-		     ;; the change in base-line makes parens unnecessary
-		     (if nc
-			 (tex (cadr x) '("^ {-\\langle ")(cons "\\rangle }" r) 'mparen 'mparen)
-             (tex (cadr x) '("^ {- ") (cons " }" r) 'mminus 'mparen))
-		     (if nc
-			 (tex x (list "^{\\langle ")(cons "\\rangle}" r) 'mparen 'mparen)
-			 (if (and (integerp x) (< x 10))
-			     (tex x (list "^")(cons "" r) 'mparen 'mparen)
-			     (tex x (list "^{")(cons "}" r) 'mparen 'mparen))
-			 )))))
-    (append l r)))
+	    (maybe-tex-mexpt-trig-like x l r)))  ; fall through if f is not trig-like
+       (t (setq l (cond ((or ($bfloatp (cadr x))
+                            (and (numberp (cadr x)) (numneedsparen (cadr x))))
+                        ; ACTUALLY THIS TREATMENT IS NEEDED WHENEVER (CAAR X) HAS GREATER BINDING POWER THAN MTIMES ...
+                        (tex (cadr x) (append l '("\\left(")) '("\\right)") lop (caar x)))
+                       (t (tex (cadr x) l nil lop (caar x))))
+               r (if (mmminusp (setq x (nformat (caddr x))))
+                     ;; the change in base-line makes parens unnecessary
+                     (if nc
+                         (tex (cadr x) '("^ {-\\langle ") (cons "\\rangle }" r) 'mparen 'mparen)
+                         (tex (cadr x) '("^ {- ") (cons " }" r) 'mminus 'mparen))
+                     (if nc
+                         (tex x (list "^{\\langle ") (cons "\\rangle}" r) 'mparen 'mparen)
+                         (if (and (integerp x) (< x 10))
+                             (tex x (list "^")(cons "" r) 'mparen 'mparen)
+                             (tex x (list "^{")(cons "}" r) 'mparen 'mparen)))))
+	  (append l r)))))
 
 (defprop mncexpt tex-mexpt tex)
 
@@ -636,7 +653,7 @@
 (defprop mquotient ("\\over") texsym)
 
 (defun tex-mquotient (x l r)
-  (if (or (null (cddr x)) (cdddr x)) (wna-err (caar x)))
+  (twoargcheck x)
   (setq l (tex (cadr x) (append l '("{{")) nil 'mparen 'mparen)
 					;the divide bar groups things
 	r (tex (caddr x) (list "}\\over{") (append '("}}")r) 'mparen 'mparen))
@@ -644,12 +661,14 @@
 
 (defprop $matrix tex-matrix tex)
 
+;; Tex dialects either offer a \pmatrix command or a pmatrix environment
+;; so we let the TeX decide which one to use.
 (defun tex-matrix(x l r) ;;matrix looks like ((mmatrix)((mlist) a b) ...)
-  (append l `("\\pmatrix{")
+  (append l `("\\ifx\\endpmatrix\\undefined\\pmatrix{\\else\\begin{pmatrix}\\fi ")
 	  (mapcan #'(lambda(y)
 		      (tex-list (cdr y) nil (list "\\cr ") "&"))
 		  (cdr x))
-	  '("}") r))
+	  '("\\ifx\\endpmatrix\\undefined}\\else\\end{pmatrix}\\fi ") r))
 
 ;; macsyma sum or prod is over integer range, not  low <= index <= high
 ;; TeX is lots more flexible .. but
@@ -895,6 +914,9 @@
 (defprop mcond tex-mcond tex)
 (defprop %mcond tex-mcond tex)
 
+(defprop %del tex-prefix tex)
+(defprop %del ("d") texsym)
+
 (defprop %derivative tex-derivative tex)
 (defun tex-derivative (x l r)
   (tex (if $derivabbrev
@@ -938,13 +960,11 @@
 				   (make-list ord :initial-element var))
 			       vars ords)))))))
 
-(defun odds(n c)
-  ;; if c=1, get the odd terms  (first, third...)
-  (cond ((null n) nil)
-	((= c 1)(cons (car n)(odds (cdr n) 0)))
-	((= c 0)(odds (cdr n) 1))))
+(defun odds (list c)
+  (ecase c
+    (1 (loop for e in list by #'cddr collect e))         ;; get the odd terms  (first, third...)
+    (0 (loop for e in (cdr list) by #'cddr collect e)))) ;; get the (second, fourth ... ) element
 
-;;
 ;; The format of MCOND expressions is documented above the definition
 ;; of DIM-MCOND in displa.lisp.  Here are some examples:
 ;;
@@ -1063,7 +1083,7 @@
   (append l (cons (format nil "\\hspace{~dmm}" (* 3 (cadr x))) r)))
 
 ;; run some code initialize file before $tex is run
-(defun $texinit(file)
+(defmfun $texinit(file)
 (declare (ignore file))
   '$done)
 
@@ -1071,7 +1091,7 @@
 ;; probably have no trouble spotting, and will generally be unnecessary, since
 ;; we anticipate almost all use of tex would be involved in inserting this
 ;; stuff into larger files that would have their own \\end or equivalent.
-(defun $texend(filename)
+(defmfun $texend(filename)
   (with-open-file (st (stripdollar filename)  :direction :output
 		      :if-exists :append :if-does-not-exist :create)
     (format st "\\end~%"))
@@ -1099,7 +1119,7 @@
 ;; Convenience function to allow user to process expression X
 ;; and get a string (TeX output for X) in return.
 
-(defun $tex1 (x) (apply #'strcat (tex x nil nil 'mparen 'mparen)))
+(defmfun $tex1 (x) (reduce #'strcat (tex x nil nil 'mparen 'mparen)))
 
 ;; Undone and trickier:
 ;; handle reserved symbols stuff, just in case someone
@@ -1112,7 +1132,7 @@
 
 ;;  The texput function was written by Barton Willis.
 
-(defun $texput (e s &optional tx)
+(defmfun $texput (e s &optional tx)
 
   (cond
     ((stringp e)
